package org.test;

import org.axonframework.commandhandling.CommandBus;
import org.axonframework.eventhandling.EventBus;
import org.axonframework.eventhandling.saga.ResourceInjector;
import org.axonframework.eventhandling.saga.repository.SagaStore;
import org.axonframework.eventsourcing.eventstore.EmbeddedEventStore;
import org.axonframework.eventsourcing.eventstore.EventStorageEngine;
import org.axonframework.eventsourcing.eventstore.EventStore;
import org.axonframework.mongo.eventhandling.saga.repository.MongoSagaStore;
import org.axonframework.mongo.eventsourcing.eventstore.DefaultMongoTemplate;
import org.axonframework.mongo.eventsourcing.eventstore.MongoEventStorageEngine;
import org.axonframework.spring.config.EnableAxon;
import org.axonframework.spring.saga.SpringResourceInjector;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;

import com.mongodb.MongoClient;

@EnableAxon
@SpringBootApplication
@EnableMongoRepositories
// @EnableVaadinServlet
public class TestApplication {

    String database = "test";

    String sagaCollection = "sagas";

    String eventStoreCollection = "events";

    String snapshotsCollection = "snapshots";

    @Autowired
    MongoClient mongoClient;

    @Autowired
    CommandBus commandBus;

    @Bean
    public EventStore eventStore() {
        return new EmbeddedEventStore(eventStorageEngine());
    }

    @Bean
    EventBus eventBus() {
        return eventStore();
    }

    @Bean
    public ResourceInjector resourceInjector() {
        return new SpringResourceInjector();
    }

    @Bean
    EventStorageEngine eventStorageEngine() {
        return new MongoEventStorageEngine(
                new DefaultMongoTemplate(mongoClient, database, eventStoreCollection, snapshotsCollection));
    }

    @Bean
    public SagaStore<Object> sagaStore() {
        return new MongoSagaStore(new org.axonframework.mongo.eventhandling.saga.repository.DefaultMongoTemplate(
                mongoClient, database, sagaCollection));
    }

    public static void main(String... args) {
        SpringApplication.run(TestApplication.class, args);
    }

}
