package org.test.domain;

import static org.axonframework.eventhandling.saga.SagaLifecycle.associateWith;
import static org.axonframework.eventhandling.saga.SagaLifecycle.end;

import javax.inject.Inject;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;
import org.axonframework.commandhandling.gateway.CommandGateway;
import org.axonframework.common.IdentifierFactory;
import org.axonframework.eventhandling.saga.SagaEventHandler;
import org.axonframework.eventhandling.saga.StartSaga;
import org.axonframework.spring.stereotype.Saga;

import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Saga
@Slf4j
@NoArgsConstructor
public class TestSaga {

    @Inject
    private CommandGateway gateway;

    @StartSaga
    @SagaEventHandler(associationProperty = "id")
    public void on(TestCreatedEvent event) {
        log.info("saga started");
        String newId = IdentifierFactory.getInstance().generateIdentifier();
        CreateTest2Command c = new CreateTest2Command(newId);
        log.info("send command: {}", c);
        associateWith("id", newId);
        gateway.send(c).whenComplete(
                (result, exception) -> log.info("Result of problematic command: {} {}", result, exception));
    }

    @SagaEventHandler(associationProperty = "id")
    public void on(Test2CreatedEvent event) {
        log.info("got event: {}", ReflectionToStringBuilder.toString(event));

        end();
    }
}
